def session_setup():
    user_local_waptconsole_ini = makepath(user_local_appdata, "waptconsole", "waptconsole.ini")
    new_local_waptconsole_ini = get_public_persistent_file("waptconsole.ini")

    new_inifile = configparser.RawConfigParser()
    new_inifile.read(new_local_waptconsole_ini)

    user_inifile = configparser.RawConfigParser()
    user_inifile.read(user_local_waptconsole_ini)

    # Comparing ini files
    sections_updated = False
    for section in new_inifile.sections():
        if section not in ["global", "waptwua"] and section not in user_inifile.sections():
            print(f"[{section}] added")
            sections_updated = True
            if not user_inifile.has_section(section):
                user_inifile.add_section(section)
            user_inifile.set(section, "repo_url", new_inifile[section]["repo_url"])
    for section in user_inifile.sections():
        if section in repos_to_remove:
            print(f"[{section}] removed")
            sections_updated = True
            user_inifile.remove_section(section)
    if sections_updated:
        print("WAPT repos updated in WAPT Console")
        user_inifile.write(open(user_local_waptconsole_ini, "w"))


from configparser import RawConfigParser, ConfigParser


def install():
    app_version_file = makepath(install_dir, "wampmanager.ini")
    inifile_readstring(app_version_file, "Config", "AboutVersion")

    inifile_readstring(app_version_file, "Config", "AboutVersion", strict=False, allow_no_value=True)


def inifile_readstring(inifilename, section, key, **kwargs):
    """
    Read a string parameter from an INI file.

    :param inifilename: Path to the INI file.
    :param section: Section name in the INI file.
    :param key: Key name in the INI file.
    :param default: Default value to return if the key is not found (optional).
    :return: The value associated with the key in the INI file or the default value.
    """
    encoding = kwargs.get("encoding", None)
    if encoding is None:
        with open(inifilename, "rb") as file:
            encoding = chardet.detect(file.read())["encoding"]

    inifile = RawConfigParser(**kwargs)
    inifile.optionxform = str
    inifile.read(inifilename, encoding=encoding)
    if inifile.has_option(section, key):
        return inifile.get(section, key)
    else:
        return kwargs.get("default", None)


def inifile_writestring(inifilename, section, key, value, **kwargs):
    """
    Write a string parameter to an INI file.

    :param inifilename: Path to the INI file.
    :param section: Section name in the INI file.
    :param key: Key name in the INI file.
    :param value: Value to write.
    """
    encoding = kwargs.get("encoding", None)
    if encoding is None:
        with open(inifilename, "rb") as file:
            encoding = chardet.detect(file.read())["encoding"]

    inifile = RawConfigParser(**kwargs)
    inifile.optionxform = str
    inifile.read(inifilename, encoding=encoding)
    if not inifile.has_section(section):
        inifile.add_section(section)
    inifile.set(section, key, value)
    with open(inifilename, "w", encoding=encoding) as file:
        inifile.write(file)
