import psutil


def isrunning(processname, full_path=False):
    """Check if a process is running.

    Args:
        processname (str): The name of the process.
        full_path (bool, optional): Whether to match the full path of the process executable.
                                    Defaults to False.

    Returns:
        bool: True if the process is running, False otherwise.

    Examples:
        >>> isrunning('explorer')
        True

        >>> isrunning('setup.exe', full_path=False)
        True

        >>> isrunning('C:\\Program Files (x86)\\Microsoft Visual Studio\\Installer\\setup.exe', full_path=True)
        False
    """
    processname = processname.lower()
    for p in psutil.process_iter():
        try:
            if full_path:
                if p.exe().lower() == processname or p.name().lower() == processname + ".exe":
                    return True
            else:
                if p.name().lower() == processname or p.name().lower() == processname + ".exe":
                    return True
        except (psutil.AccessDenied, psutil.NoSuchProcess):
            pass
    return False
