# -*- coding: utf-8 -*-
from setuphelpers import *
import json
import platform

# {731F6BAA-A986-45A4-8936-7C3AAAAA760B}      Teams Machine-Wide Installer        1.3.0.12058         MsiExec.exe /I{731F6BAA-A986-45A4-8936-7C3AAAAA760B}

uninstallkey = []

# Installation procedure: https://docs.microsoft.com/fr-fr/microsoftteams/teams-for-vdi#install-or-update-the-teams-desktop-app-on-vdi

# Defining variables
bin_name_string = "Teams_windows_%s_x64.msi"
bin_name_temp = "Teams_windows_x64.msi"
url_dl = "https://teams.microsoft.com/downloads/desktopurl?env=production&plat=windows&arch=x64&managedInstaller=true&download=true"
silent_args = "ALLUSER=1"  # OPTIONS="noAutoStart=true" ALLUSER=1 ALLUSERS=1
app_name_short = "Teams"
app_dir = makepath(programfiles32, "Microsoft", "Teams")
sys_json_setup = makepath(app_dir, "setup.json")
json_setup_content = json.loads('{"noAutoStart":true,"--exeName":"Teams.exe"}')


def install():
    # Initializing variables
    package_version = control.version.split("-")[0]
    bin_name = bin_name_string % package_version
    app_name = control.name

    # Uninstalling older versions if found
    for uninstall in installed_softwares(app_name_short):
        if Version(uninstall["version"]) < Version(package_version):
            print("Older %s version found (%s)" % (app_name, uninstall["version"]))
            print("Removing: %s" % uninstall["name"])
            run(uninstall_cmd(uninstall["key"]))

    # Force uninstalling to make sure that we are using system-wide installation
    for uninstall in installed_softwares(app_name_short):
        print("Removing: %s" % uninstall["name"])
        run(uninstall_cmd(uninstall["key"]))

    # Adding fake WVD/VDI RegKey
    # Un/comment if needed
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.\VMware VDM\Agent", "", "")
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA", "", "")
    registry_set(
        HKEY_LOCAL_MACHINE,
        r"SOFTWARE\Microsoft\Teams",
        "IsWVDEnvironment",
        1,
        type=REG_DWORD,
    )

    # Installing the package system wide
    install_msi_if_needed(
        bin_name,
        properties=silent_args,
        min_version=package_version,
        accept_returncodes=[0, 3010, 1605, 1614, 1641],
        timeout=900,
    )

    # Disabling AutoStart
    write_json(sys_json_setup, json_setup_content)


def uninstall():
    # Cleaning fake WVD/VDI RegKey properly
    # Un/comment if needed
    if reg_value_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Teams", "IsWVDEnvironment"):
        registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Teams", "IsWVDEnvironment")
    if reg_key_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc."):
        registry_deletekey(
            HKEY_LOCAL_MACHINE,
            r"SOFTWARE\VMware, Inc.\VMware VDM\Agent",
            "",
            force=False,
        )
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.\VMware VDM", "", force=False)
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.", "", force=False)
    if reg_key_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA"):
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA", "", force=False)

    # Removing remaining files
    if isdir(app_dir):
        remove_tree(app_dir)


def session_setup():
    # Initializing variables
    user_conf_dir = makepath(application_data, "Microsoft", "Teams")
    json_config = makepath(user_conf_dir, "desktop-config.json")
    json_settings = makepath(user_conf_dir, "settings.json")

    user_app_dir = makepath(user_local_appdata, "Microsoft", "Teams")
    user_app_path = makepath(user_app_dir, "current", "Teams.exe")
    user_app_updater = makepath(user_app_dir, "Update.exe")
    user_app_version = registry_readstring(
        HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Teams",
        "DisplayVersion",
    )

    user_app_progdata_dir = makepath("C:", "ProgramData", get_current_user(), "Microsoft", "Teams")
    user_app_progdata_path = makepath(user_app_progdata_dir, "current", "Teams.exe")
    user_app_progdata_updater = makepath(user_app_progdata_dir, "Update.exe")

    package_version = control.version.split("-")[0]
    user_silent_args_uninst = "--uninstall -s"

    # Killing app before making changes
    killalltasks("Teams.exe")

    # Uninstalling from user environment (procedure: https://docs.microsoft.com/microsoftteams/msi-deployment)
    if isfile(user_app_path):
        print("Uninstalling Microsoft Teams from user environment")
        run('"%s" %s' % (user_app_updater, user_silent_args_uninst))
    if isdir(user_app_dir):
        remove_tree(user_app_dir)
    if isfile(user_app_progdata_path):
        print("Uninstalling Microsoft Teams from user ProgramData environment")
        run('"%s" %s' % (user_app_progdata_updater, user_silent_args_uninst))
    if isdir(user_app_progdata_dir):
        remove_tree(user_app_progdata_dir)

    if reg_value_exists(
        HKEY_CURRENT_USER,
        r"Software\Microsoft\Office\Teams",
        "PreventInstallationFromMsi",
    ):
        registry_delete(
            HKEY_CURRENT_USER,
            r"Software\Microsoft\Office\Teams",
            "PreventInstallationFromMsi",
        )

    # Define user settings (procedure: https://techcommunity.microsoft.com/t5/microsoft-teams/how-to-disable-check-for-updates-in-microsoft-teams/m-p/54644)
    print("Setting up user settings")
    if not isdir(user_conf_dir):
        mkdirs(user_conf_dir)

    # desktop-config.json
    if isfile(json_config):
        json_config_content = load_json(json_config)
    else:
        json_config_content = json.loads('{"appPreferenceSettings":{}}')

    json_config_content["appPreferenceSettings"]["openAtLogin"] = False
    json_config_content["appPreferenceSettings"]["openAsHidden"] = False
    # json_config_content['appPreferenceSettings']['isForeGround'] = False
    write_json(json_config, json_config_content)

    # settings.json
    if isfile(json_settings):
        json_settings_content = load_json(json_settings)
    else:
        json_settings_content = json.loads('{"baseSettings":{},"initializedSettings":{}}')

    json_settings_content["baseSettings"]["enableBlockSilentUpdate"] = True
    json_settings_content["initializedSettings"]["disableAllUsersAutoUpdate"] = True
    write_json(json_settings, json_settings_content)


def load_json(json_filename):
    with open(json_filename) as json_file:
        data = json.load(json_file)
    return data


def write_json(json_filename, data):
    with open(json_filename, "w") as outfile:
        json.dump(data, outfile, sort_keys=True, indent=4, default=str)


def update_package():
    print("Download/Update package content from upstream binary sources")

    # Getting proxy informations from WAPT settings
    proxy = {}
    if platform.system() == "Windows" and isfile(makepath(user_local_appdata(), "waptconsole", "waptconsole.ini")):
        proxywapt = inifile_readstring(
            makepath(user_local_appdata(), "waptconsole", "waptconsole.ini"),
            "global",
            "http_proxy",
        )
        if proxywapt:
            proxy = {"http": proxywapt, "https": proxywapt}

    # Initializing variables
    latest_bin = bin_name_temp

    # Downloading latest binaries
    if isfile(bin_name_temp):
        remove_file(bin_name_temp)
    if not isfile(latest_bin):
        print("Downloading: " + latest_bin)
        wget(url_dl, latest_bin, proxies=proxy)

        # Get version from file
        version_from_file = get_msi_properties(latest_bin)["ProductVersion"]
        version = version_from_file
        old_latest_bin = latest_bin
        latest_bin = bin_name_string % version
        if isfile(latest_bin):
            remove_file(latest_bin)
        os.rename(old_latest_bin, latest_bin)

        # Change version of the package
        pe = PackageEntry().load_control_from_wapt(".")
        pe.version = "%s-%s" % (version, int(pe.version.split("-", 1)[1]) + 1)
        pe.save_control_to_wapt(".")
        print("Changing version to " + pe.version + " in WAPT\\control")
        print("Update package done. You can now build-upload your package")
    else:
        print("This package is already up-to-date")

    # Deleting outdated binaries
    for bin_in_dir in glob.glob("*.exe") or glob.glob("*.msi") or glob.glob("*.zip"):
        if bin_in_dir != latest_bin:
            print("Outdated binary: " + bin_in_dir + " Deleted")
            remove_file(bin_in_dir)
