# -*- coding: utf-8 -*-
from setuphelpers import *

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_string = "mRemoteNG-Installer-%s.msi"
silent_args = "/quiet /norestart"


def install():
    print("installing %s" % control.asrequirement())

    # Specific app values
    package_version = control.version.split("-", 1)[0]

    # Getting the used storage on programfiles before installation (place it on the top)
    get_disk_free_space_before = get_disk_free_space(programfiles)

    # Installing the package
    install_msi_if_needed(bin_name_string % package_version, min_version=package_version)

    # Return used storage of the program. (place it on the bottom)
    get_disk_free_space_after = get_disk_free_space(programfiles)
    free_space_after_diff = get_disk_free_space_before - get_disk_free_space_after
    print("Storage used: " + str(free_space_after_diff))


def session_setup():
    print("Session setup for %s" % control.asrequirement())
    print("Disabling: update check")

    # Specific app values
    package_version = control.version.split("-", 1)[0]

    # C:\Users\jpadmin\AppData\Local\mRemoteNG\mRemoteNG.exe_Url_pjpxdehxpaaorqg2thmuhl11a34i3ave\1.76.20.24615
    app_local_appdata = makepath(user_local_appdata, "mRemoteNG", "*")

    for prof_in_dir in glob.iglob(app_local_appdata):
        profile_app_dir = prof_in_dir + "/" + package_version
        conf_file_loc = profile_app_dir + "/" + "user.config"
    # print(profile_app_dir)
    # print(conf_file_loc)

    if isfile(conf_file_loc):
        import xml.etree.ElementTree as ET

        xml_data_conf_tree = ET.parse(conf_file_loc)
        xml_data_conf = xml_data_conf_tree.getroot()

        # print(xml_data_conf_tree)

        for conf in xml_data_conf.iter("setting"):
            if conf.get("name") == "CheckForUpdatesOnStartup":
                for children in conf.getchildren():
                    children.text = "False"
                    xml_data_conf_tree.write(conf_file_loc, encoding="utf-8")


def update_package():
    print("Download/Update package content from upstream binary sources")
    from waptpackage import PackageEntry
    import json

    # Getting proxy informations from WAPT settings
    proxy = {}
    if isfile(makepath(application_data(), "waptconsole", "waptconsole.ini")):
        proxywapt = inifile_readstring(
            makepath(user_local_appdata(), "waptconsole", "waptconsole.ini"),
            "global",
            "http_proxy",
        )
        if proxywapt:
            proxy = {"http": proxywapt, "https": proxywapt}

    # Specific app values
    app_name = control.name
    url_api = "https://api.github.com/repos/mRemoteNG/mRemoteNG/releases/latest"

    # Getting latest informations from Github API
    json_load = json.loads(wgets(url_api, proxies=proxy))
    for download in json_load["assets"]:
        if download["browser_download_url"].endswith(".msi"):
            url_dl = download["browser_download_url"]
            break

    version = (url_dl.split("-")[-1]).replace(".msi", "")
    latest_bin = bin_name_string % version

    print("Latest " + app_name + " version is: " + version)
    print("Download url is: " + url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: " + latest_bin)
        wget(url_dl, latest_bin, proxies=proxy)

        # Change version of the package
        print("Writing " + version + "-0 in WAPT\\control")
        pe = PackageEntry()
        pe.load_control_from_wapt(os.getcwd())

        pe.version = version + "-0"
        pe.save_control_to_wapt(os.getcwd())
        print("Update package done. You can now build-upload your package")
    else:
        print("This package is already up-to-date")

    # Deleting outdated binaries
    for bin_in_dir in glob.glob("*.exe") or glob.glob("*.msi"):
        if bin_in_dir != latest_bin:
            print("Outdated binary: " + bin_in_dir + " Deleted")
            remove_file(bin_in_dir)


if __name__ == "__main__":
    update_package()
